@@ -124,11 +124,16 @@ int intro(struct notcurses* nc){
     }
   }
   ncplane_off_styles(ncp, NCSTYLE_BOLD);
-  const wchar_t wstr[] = L"▏▁ ▂ ▃ ▄ ▅ ▆ ▇ █ █ ▇ ▆ ▅ ▄ ▃ ▂ ▁▕";
-  if(ncplane_putwstr_aligned(ncp, rows / 2 - 6, NCALIGN_CENTER, wstr) < 0){
+  const wchar_t nwstr[] = L"▏▁ ▂ ▃ ▄ ▅ ▆ ▇ █ █ ▇ ▆ ▅ ▄ ▃ ▂ ▁▕";
+  if(ncplane_putwstr_aligned(ncp, rows / 2 - 6, NCALIGN_CENTER, nwstr) < 0){
     return -1;
   }
-  const wchar_t iwstr[] = L"▏█ ▇ ▆ ▅ ▄ ▃ ▂ ▁ ▁ ▂ ▃ ▄ ▅ ▆ ▇ █▕";
+  const wchar_t* iwstr;
+  if(notcurses_cansextant(nc)){
+    iwstr = L"▏▔ 🮂 🮃 ▀ 🮄 🮅 🮆 █ █ 🮆 🮅 🮄 ▀ 🮃 🮂 ▔▕";
+  }else{
+    iwstr = L"▏█ ▇ ▆ ▅ ▄ ▃ ▂ ▁ ▁ ▂ ▃ ▄ ▅ ▆ ▇ █▕";
+  }
   if(ncplane_putwstr_aligned(ncp, rows / 2 + 1, NCALIGN_CENTER, iwstr) < 0){
     return -1;
   }
@@ -142,7 +147,7 @@ int intro(struct notcurses* nc){
     ncplane_off_styles(ncp, NCSTYLE_BLINK); // heh FIXME replace with pulse
   }
   struct timespec now;
-  clock_gettime(CLOCK_MONOTONIC_RAW, &now);
+  clock_gettime(CLOCK_MONOTONIC, &now);
   uint64_t deadline = timespec_to_ns(&now) + timespec_to_ns(&demodelay) * 2;
   int flipmode = 0;
   do{
@@ -153,7 +158,7 @@ int intro(struct notcurses* nc){
     struct timespec iter;
     timespec_div(&demodelay, 10, &iter);
     demo_nanosleep(nc, &iter);
-    clock_gettime(CLOCK_MONOTONIC_RAW, &now);
+    clock_gettime(CLOCK_MONOTONIC, &now);
   }while(timespec_to_ns(&now) < deadline);
   if(!notcurses_canfade(nc)){
     return 0;
