@@ -264,23 +264,25 @@ typedef struct tinfo {
   char* smkx;     // enter keypad transmit mode (keypad_xmit)
   char* rmkx;     // leave keypad transmit mode (keypad_local)
   char* getm;     // get mouse events
+  char* smcup;    // enter alternate mode
+  char* rmcup;    // restore primary mode
   bool RGBflag;   // "RGB" flag for 24bpc truecolor
   bool CCCflag;   // "CCC" flag for palette set capability
   bool BCEflag;   // "BCE" flag for erases with background color
   bool AMflag;    // "AM" flag for automatic movement to next line
-  char* smcup;    // enter alternate mode
-  char* rmcup;    // restore primary mode
+  bool utf8;      // are we using utf-8 encoding, as hoped?
 
   // kitty interprets an RGB background that matches the default background
   // color *as* the default background, meaning it'll be translucent if
   // background_opaque is in use. detect this, and avoid the default if so.
   // bg_collides_default is either 0x0000000 or 0x1RRGGBB.
   uint32_t bg_collides_default;
-  bool sextants;  // do we have Unicode 13 sextant support?
+  bool sextants;  // do we have (good, vetted) Unicode 13 sextant support?
+  bool braille;   // do we have Braille support? (linux console does not)
 } tinfo;
 
 typedef struct ncinputlayer {
-  FILE* ttyinfp;  // FILE* for processing input
+  int ttyinfd;  // file descriptor for processing input
   unsigned char inputbuf[BUFSIZ];
   // we keep a wee ringbuffer of input queued up for delivery. if
   // inputbuf_occupied == sizeof(inputbuf), there is no room. otherwise, data
@@ -304,7 +306,6 @@ typedef struct ncdirect {
   uint64_t channels;         // current channels
   uint16_t stylemask;        // current styles
   ncinputlayer input;        // input layer; we're in cbreak mode
-  bool utf8;                 // are we using utf-8 encoding, as hoped?
   struct termios tpreserved; // terminal state upon entry
   // some terminals (e.g. kmscon) return cursor coordinates inverted from the
   // typical order. we detect it the first time ncdirect_cursor_yx() is called.
@@ -360,7 +361,6 @@ typedef struct notcurses {
   int loglevel;
   palette256 palette; // 256-indexed palette can be used instead of/with RGB
   bool palette_damage[NCPALETTESIZE];
-  bool utf8;      // are we using utf-8 encoding, as hoped?
   bool libsixel;  // do we have Sixel support?
 } notcurses;
 
@@ -505,7 +505,7 @@ rgb_greyscale(int r, int g, int b){
 }
 
 static inline int
-tty_emit(const char* name __attribute__ ((unused)), const char* seq, int fd){
+tty_emit(const char* seq, int fd){
   if(!seq){
     return -1;
   }
@@ -523,27 +523,25 @@ tty_emit(const char* name __attribute__ ((unused)), const char* seq, int fd){
     }
   }while(written < slen);
   if(written < slen){
-//fprintf(stderr, "Error emitting %zub %s escape (%s)\n", strlen(seq), name, strerror(errno));
+//fprintf(stderr, "Error emitting %zub escape (%s)\n", strlen(seq), strerror(errno));
     return -1;
   }
   return 0;
 }
 
 static inline int
-term_emit(const char* name __attribute__ ((unused)), const char* seq,
-          FILE* out, bool flush){
+term_emit(const char* seq, FILE* out, bool flush){
   if(!seq){
     return -1;
   }
   if(fputs(seq, out) == EOF){
-//fprintf(stderr, "Error emitting %zub %s escape (%s)\n", strlen(seq), name, strerror(errno));
+//fprintf(stderr, "Error emitting %zub escape (%s)\n", strlen(seq), strerror(errno));
     return -1;
   }
   if(flush){
     while(fflush(out) == EOF){
       if(errno != EAGAIN){
-        fprintf(stderr, "Error flushing after %zub %s sequence (%s)\n",
-                strlen(seq), name, strerror(errno));
+        fprintf(stderr, "Error flushing after %zub sequence (%s)\n", strlen(seq), strerror(errno));
         return -1;
       }
     }
@@ -556,7 +554,7 @@ term_bg_palindex(const notcurses* nc, FILE* out, unsigned pal){
   if(nc->tcache.setab == NULL){
     return 0;
   }
-  return term_emit("setab", tiparm(nc->tcache.setab, pal), out, false);
+  return term_emit(tiparm(nc->tcache.setab, pal), out, false);
 }
 
 static inline int
@@ -564,7 +562,7 @@ term_fg_palindex(const notcurses* nc, FILE* out, unsigned pal){
   if(nc->tcache.setaf == NULL){
     return 0;
   }
-  return term_emit("setaf", tiparm(nc->tcache.setaf, pal), out, false);
+  return term_emit(tiparm(nc->tcache.setaf, pal), out, false);
 }
 
 static inline const char*
@@ -871,10 +869,11 @@ static inline void
 cell_set_blitquadrants(nccell* c, unsigned tl, unsigned tr, unsigned bl, unsigned br){
   // FIXME want a static assert that these four constants OR together to
   // equal CELL_BLITTERSTACK_MASK, bah
-  c->channels |= (tl ? 0x8000000000000000ull : 0);
-  c->channels |= (tr ? 0x0400000000000000ull : 0);
-  c->channels |= (bl ? 0x0200000000000000ull : 0);
-  c->channels |= (br ? 0x0100000000000000ull : 0);
+  uint64_t newval = (tl ? 0x8000000000000000ull : 0) |
+                    (tr ? 0x0400000000000000ull : 0) |
+                    (bl ? 0x0200000000000000ull : 0) |
+                    (br ? 0x0100000000000000ull : 0);
+  c->channels = ((c->channels & ~CELL_BLITTERSTACK_MASK) | newval);
 }
 
 // Destroy a plane and all its bound descendants.
@@ -953,36 +952,35 @@ egc_rtl(const char* egc, int* bytes){
 }
 
 // lowest level of cell+pool setup. if the EGC changes the output to RTL, it
-// must be suffixed with a LTR-forcing character by now, and
-// CELL_NOBACKGROUND_MASK ought be set however it's going to be set.
+// must be suffixed with a LTR-forcing character by now. The four bits of
+// CELL_BLITTERSTACK_MASK ought already be initialized. If gcluster is four
+// bytes or fewer, this function cannot fail.
 static inline int
 pool_blit_direct(egcpool* pool, nccell* c, const char* gcluster, int bytes, int cols){
   pool_release(pool, c);
   if(bytes < 0 || cols < 0){
     return -1;
   }
+  c->width = cols;
   if(bytes <= 4){
     c->gcluster = 0;
     memcpy(&c->gcluster, gcluster, bytes);
-    return bytes;
+  }else{
+    int eoffset = egcpool_stash(pool, gcluster, bytes);
+    if(eoffset < 0){
+      return -1;
+    }
+    set_gcluster_egc(c, eoffset);
   }
-  int eoffset = egcpool_stash(pool, gcluster, bytes);
-  if(eoffset < 0){
-    return -1;
-  }
-  set_gcluster_egc(c, eoffset);
   return bytes;
 }
 
+// Do an RTL-check, reset the quadrant occupancy bits, and pass the cell down to
+// pool_blit_direct().
 static inline int
 pool_load_direct(egcpool* pool, nccell* c, const char* gcluster, int bytes, int cols){
   char* rtl = NULL;
-  c->width = cols;
-  if(bytes == 3 && memcmp(gcluster, "\xe2\x96\x88", 4) == 0){
-    c->channels |= CELL_NOBACKGROUND_MASK;
-  }else{
-    c->channels &= ~CELL_NOBACKGROUND_MASK;
-  }
+  c->channels &= ~CELL_NOBACKGROUND_MASK;
   if(bytes >= 0){
     rtl = egc_rtl(gcluster, &bytes); // checks for RTL and adds U+200E if so
   }
@@ -999,13 +997,6 @@ cell_load_direct(ncplane* n, nccell* c, const char* gcluster, int bytes, int col
   return pool_load_direct(&n->pool, c, gcluster, bytes, cols);
 }
 
-static inline int
-pool_load(egcpool* pool, nccell* c, const char* gcluster){
-  int cols;
-  int bytes = utf8_egc_len(gcluster, &cols);
-  return pool_load_direct(pool, c, gcluster, bytes, cols);
-}
-
 // increment y by 1 and rotate the framebuffer up one line. x moves to 0.
 void scroll_down(ncplane* n);
 
