@@ -39,13 +39,13 @@ void notcurses_version_components(int* major, int* minor, int* patch, int* tweak
 static int
 reset_term_attributes(notcurses* nc){
   int ret = 0;
-  if(nc->tcache.op && term_emit("op", nc->tcache.op, nc->ttyfp, false)){
+  if(nc->tcache.op && term_emit(nc->tcache.op, nc->ttyfp, false)){
     ret = -1;
   }
-  if(nc->tcache.sgr0 && term_emit("sgr0", nc->tcache.sgr0, nc->ttyfp, false)){
+  if(nc->tcache.sgr0 && term_emit(nc->tcache.sgr0, nc->ttyfp, false)){
     ret = -1;
   }
-  if(nc->tcache.oc && term_emit("oc", nc->tcache.oc, nc->ttyfp, true)){
+  if(nc->tcache.oc && term_emit(nc->tcache.oc, nc->ttyfp, true)){
     ret = -1;
   }
   ret |= notcurses_mouse_disable(nc);
@@ -63,10 +63,10 @@ notcurses_stop_minimal(void* vnc){
   // they apply to the screen (alternate or otherwise) we're actually using.
   ret |= reset_term_attributes(nc);
   if(nc->ttyfd >= 0){
-    if(nc->tcache.rmcup && tty_emit("rmcup", nc->tcache.rmcup, nc->ttyfd)){
+    if(nc->tcache.rmcup && tty_emit(nc->tcache.rmcup, nc->ttyfd)){
       ret = -1;
     }
-    if(nc->tcache.cnorm && tty_emit("cnorm", nc->tcache.cnorm, nc->ttyfd)){
+    if(nc->tcache.cnorm && tty_emit(nc->tcache.cnorm, nc->ttyfd)){
       ret = -1;
     }
     ret |= tcsetattr(nc->ttyfd, TCSANOW, &nc->tpreserved);
@@ -685,8 +685,7 @@ int ncplane_genocide(ncplane *ncp){
 }
 
 static int
-make_nonblocking(FILE* fp){
-  int fd = fileno(fp);
+make_nonblocking(int fd){
   if(fd < 0){
     return -1;
   }
@@ -812,7 +811,7 @@ init_banner(const notcurses* nc){
       fprintf(stderr, "\n Warning! Encoding is not UTF-8; output may be degraded.\n");
     }
     if(nc->tcache.sgr0){
-      term_emit("sgr0", nc->tcache.sgr0, stderr, true);
+      term_emit(nc->tcache.sgr0, stderr, true);
     }
   }
 }
@@ -887,7 +886,7 @@ get_tty_fd(notcurses* nc, FILE* ttyfp){
 int ncinputlayer_init(ncinputlayer* nilayer, FILE* infp){
   setbuffer(infp, NULL, 0);
   nilayer->inputescapes = NULL;
-  nilayer->ttyinfp = infp;
+  nilayer->ttyinfd = fileno(infp);
   if(prep_special_keys(nilayer)){
     return -1;
   }
@@ -945,10 +944,11 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
     init_lang(ret);
   }
   const char* encoding = nl_langinfo(CODESET);
+  bool utf8;
   if(encoding && !strcmp(encoding, "UTF-8")){
-    ret->utf8 = true;
+    utf8 = true;
   }else if(encoding && (!strcmp(encoding, "ANSI_X3.4-1968") || !strcmp(encoding, "US-ASCII"))){
-    ret->utf8 = false;
+    utf8 = false;
   }else{
     fprintf(stderr, "Encoding (\"%s\") was neither ANSI_X3.4-1968 nor UTF-8, refusing to start\n Did you call setlocale()?\n",
             encoding ? encoding : "none found");
@@ -1028,12 +1028,13 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   if(interrogate_terminfo(&ret->tcache, shortname_term)){
     goto err;
   }
+  ret->tcache.utf8 = utf8;
   warn_terminfo(ret, &ret->tcache);
   reset_term_attributes(ret);
   if(ncinputlayer_init(&ret->input, stdin)){
     goto err;
   }
-  if(make_nonblocking(ret->input.ttyinfp)){
+  if(make_nonblocking(ret->input.ttyinfd)){
     goto err;
   }
   // Neither of these is supported on e.g. the "linux" virtual console.
@@ -1049,11 +1050,11 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
     goto err;
   }
   if(ret->ttyfd >= 0){
-    if(ret->tcache.smkx && tty_emit("smkx", ret->tcache.smkx, ret->ttyfd)){
+    if(ret->tcache.smkx && tty_emit(ret->tcache.smkx, ret->ttyfd)){
       free_plane(ret->stdplane);
       goto err;
     }
-    if(ret->tcache.civis && tty_emit("civis", ret->tcache.civis, ret->ttyfd)){
+    if(ret->tcache.civis && tty_emit(ret->tcache.civis, ret->ttyfd)){
       free_plane(ret->stdplane);
       goto err;
     }
@@ -1067,19 +1068,19 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   // flush on the switch to alternate screen, lest initial output be swept away
   if(ret->ttyfd >= 0){
     if(ret->tcache.smcup){
-      if(tty_emit("smcup", ret->tcache.smcup, ret->ttyfd)){
+      if(tty_emit(ret->tcache.smcup, ret->ttyfd)){
         free_plane(ret->stdplane);
         goto err;
       }
       // explicit clear even though smcup *might* clear
-      if(tty_emit("clear", ret->tcache.clearscr, ret->ttyfd)){
+      if(tty_emit(ret->tcache.clearscr, ret->ttyfd)){
         notcurses_refresh(ret, NULL, NULL);
       }
     }else if(!(opts->flags & NCOPTION_NO_ALTERNATE_SCREEN)){
       // if they expected the alternate screen, but we didn't have one to
       // offer, at least clear the screen. try using "clear"; if that doesn't
       // fly, use notcurses_refresh() to force a clearing via iterated writes.
-      if(tty_emit("clear", ret->tcache.clearscr, ret->ttyfd)){
+      if(tty_emit(ret->tcache.clearscr, ret->ttyfd)){
         notcurses_refresh(ret, NULL, NULL);
       }
     }
@@ -1141,6 +1142,12 @@ int notcurses_stop(notcurses* nc){
     if(nc->rstate.mstreamfp){
       fclose(nc->rstate.mstreamfp);
     }
+    // if we were not using the alternate screen, our cursor's wherever we last
+    // wrote. move it to the bottom left of the screen.
+    if(!nc->tcache.smcup){
+      tty_emit(tiparm(nc->tcache.hpa, 0), nc->ttyfd);
+      tty_emit(tiparm(nc->tcache.vpa, nc->lfdimy - 1), nc->ttyfd);
+    }
     if(nc->ttyfd >= 0){
       ret |= close(nc->ttyfd);
     }
@@ -1446,7 +1453,9 @@ void scroll_down(ncplane* n){
 }
 
 int cell_load(ncplane* n, nccell* c, const char* gcluster){
-  return pool_load(&n->pool, c, gcluster);
+  int cols;
+  int bytes = utf8_egc_len(gcluster, &cols);
+  return pool_load_direct(&n->pool, c, gcluster, bytes, cols);
 }
 
 // where the magic happens. write the single EGC completely described by |egc|,
@@ -1457,9 +1466,10 @@ int cell_load(ncplane* n, nccell* c, const char* gcluster){
 static inline int
 ncplane_put(ncplane* n, int y, int x, const char* egc, int cols,
             uint16_t stylemask, uint64_t channels, int bytes){
-  // if scrolling is enabled, check *before ncplane_cursor_move_yx()* whether
-  // we're past the end of the line, and move to the next line if so. if x or y
-  // are specified, however, we must always try to print there.
+  // FIXME reject any control or space characters here--should be iswgraph()
+  // check *before ncplane_cursor_move_yx()* whether we're past the end of the
+  // line. if scrolling is enabled, move to the next line if so. if x or y are
+  // specified, we must always try to print at exactly that location.
   if(x != -1){
     if(x + cols > n->lenx){
       logerror(ncplane_notcurses(n), "Target x %d + %d cols >= length %d\n", x, cols, n->lenx);
@@ -1479,11 +1489,8 @@ ncplane_put(ncplane* n, int y, int x, const char* egc, int cols,
   // FIXME don't we need to check here for wide character on edge (though our
   // docs currently claim that a wide char on edge is allowed...)?
   if(*egc == '\n'){
-    if(n->scrolling){
-      scroll_down(n);
-      return 0;
-    }
-    // FIXME isn't it an error if scrolling is disabled?
+    scroll_down(n);
+    return 0;
   }
   // A wide character obliterates anything to its immediate right (and marks
   // that cell as wide). Any character placed atop one half of a wide character
@@ -2025,7 +2032,7 @@ ncplane* ncplane_above(ncplane* n){
 #define SET_SGR_MODE_MOUSE    "1006"
 int notcurses_mouse_enable(notcurses* n){
   if(n->ttyfd >= 0){
-    return tty_emit("mouse", ESC "[?" SET_BTN_EVENT_MOUSE ";"
+    return tty_emit(ESC "[?" SET_BTN_EVENT_MOUSE ";"
                     /*SET_FOCUS_EVENT_MOUSE ";" */SET_SGR_MODE_MOUSE "h",
                     n->ttyfd);
   }
@@ -2036,7 +2043,7 @@ int notcurses_mouse_enable(notcurses* n){
 // the sequences 1000 etc?
 int notcurses_mouse_disable(notcurses* n){
   if(n->ttyfd >= 0){
-    return tty_emit("mouse", ESC "[?" SET_BTN_EVENT_MOUSE ";"
+    return tty_emit(ESC "[?" SET_BTN_EVENT_MOUSE ";"
                     /*SET_FOCUS_EVENT_MOUSE ";" */SET_SGR_MODE_MOUSE "l",
                     n->ttyfd);
   }
@@ -2044,11 +2051,15 @@ int notcurses_mouse_disable(notcurses* n){
 }
 
 bool notcurses_canutf8(const notcurses* nc){
-  return nc->utf8;
+  return nc->tcache.utf8;
 }
 
 bool notcurses_cansextant(const notcurses* nc){
-  return nc->tcache.sextants && nc->utf8;
+  return nc->tcache.sextants && nc->tcache.utf8;
+}
+
+bool notcurses_canbraille(const notcurses* nc){
+  return nc->tcache.braille && nc->tcache.utf8;
 }
 
 bool notcurses_canfade(const notcurses* nc){
@@ -2409,11 +2420,11 @@ int notcurses_lex_margins(const char* op, notcurses_options* opts){
 }
 
 int notcurses_inputready_fd(notcurses* n){
-  return fileno(n->input.ttyinfp);
+  return n->input.ttyinfd;
 }
 
 int ncdirect_inputready_fd(ncdirect* n){
-  return fileno(n->input.ttyinfp);
+  return n->input.ttyinfd;
 }
 
 uint32_t* ncplane_rgba(const ncplane* nc, ncblitter_e blit,
