@@ -66,7 +66,7 @@ auto perframe(struct ncvisual* ncv, struct ncvisual_options* vopts,
   }
   if(!marsh->quiet){
     stdn->printf(0, NCAlign::Left, "frame %06d\u2026 (%s)", marsh->framecount,
-                 notcurses_str_blitter(marsh->blitter));
+                 notcurses_str_blitter(vopts->blitter));
   }
   char* subtitle = ncvisual_subtitle(ncv);
   if(subtitle){
@@ -273,6 +273,9 @@ auto main(int argc, char** argv) -> int {
   auto nonopt = handle_opts(argc, argv, ncopts, &quiet, &timescale, &scalemode,
                             &blitter, &displaytime, &loop);
   ncopts.flags |= NCOPTION_INHIBIT_SETLOCALE;
+  if(quiet){
+    ncopts.flags |= NCOPTION_SUPPRESS_BANNERS;
+  }
   NotCurses nc{ncopts};
   if(!nc.can_open_images()){
     nc.stop();
